@@ -1,26 +1,28 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """ PortalFolder: CMF-enabled Folder objects.
 
-$Id: PortalFolder.py,v 1.37.4.5 2002/08/04 04:04:44 efge Exp $
+$Id: PortalFolder.py,v 1.45.4.7 2004/07/28 17:05:37 shh Exp $
 """
 
 import sys
 import Globals, re, base64, marshal, string
-import CMFCorePermissions
 
-from CMFCorePermissions import View, ManageProperties, ListFolderContents
-from CMFCorePermissions import AddPortalFolders, AddPortalContent
+from CMFCorePermissions import View
+from CMFCorePermissions import ManageProperties
+from CMFCorePermissions import ListFolderContents
+from CMFCorePermissions import AddPortalFolders
+from CMFCorePermissions import AddPortalContent
 from CMFCatalogAware import CMFCatalogAware
 from OFS.Folder import Folder
 from OFS.ObjectManager import REPLACEABLE
@@ -30,38 +32,43 @@ from Acquisition import aq_parent, aq_inner, aq_base
 from DynamicType import DynamicType
 from utils import getToolByName, _checkPermission
 
-factory_type_information = ( { 'id'             : 'Folder'
-                             , 'meta_type'      : 'Portal Folder'
-                             , 'description'    : """\
-Use folders to put content in categories."""
-                             , 'icon'           : 'folder_icon.gif'
-                             , 'product'        : 'CMFCore'
-                             , 'factory'        : 'manage_addPortalFolder'
-                             , 'filter_content_types' : 0
-                             , 'immediate_view' : 'folder_edit_form'
-                             , 'actions'        :
-                                ( { 'id'            : 'view'
-                                  , 'name'          : 'View'
-                                  , 'action'        : ''
-                                  , 'permissions'   : (View,)
-                                  , 'category'      : 'folder'
-                                  }
-                                , { 'id'            : 'edit'
-                                  , 'name'          : 'Edit'
-                                  , 'action'        : 'folder_edit_form'
-                                  , 'permissions'   : (ManageProperties,)
-                                  , 'category'      : 'folder'
-                                  }
-                                , { 'id'            : 'localroles'
-                                  , 'name'          : 'Local Roles'
-                                  , 'action'        : 'folder_localrole_form'
-                                  , 'permissions'   : (ManageProperties,)
-                                  , 'category'      : 'folder'
-                                  }
-                                )
-                             }
-                           ,
-                           )
+try:
+    from zExceptions import BadRequest
+except ImportError:
+    BadRequest = 'BadRequest'
+
+factory_type_information = (
+  { 'id'             : 'Folder'
+  , 'meta_type'      : 'Portal Folder'
+  , 'description'    : """ Use folders to put content in categories."""
+  , 'icon'           : 'folder_icon.gif'
+  , 'product'        : 'CMFCore'
+  , 'factory'        : 'manage_addPortalFolder'
+  , 'filter_content_types' : 0
+  , 'immediate_view' : 'folder_edit_form'
+  , 'actions'        : ( { 'id'            : 'view'
+                         , 'name'          : 'View'
+                         , 'action': 'string:${object_url}'
+                         , 'permissions'   : (View,)
+                         , 'category'      : 'folder'
+                         }
+                       , { 'id'            : 'edit'
+                         , 'name'          : 'Edit'
+                         , 'action': 'string:${object_url}/folder_edit_form'
+                         , 'permissions'   : (ManageProperties,)
+                         , 'category'      : 'folder'
+                         }
+                       , { 'id'            : 'localroles'
+                         , 'name'          : 'Local Roles'
+                         , 'action':
+                                  'string:${object_url}/folder_localrole_form'
+                         , 'permissions'   : (ManageProperties,)
+                         , 'category'      : 'folder'
+                         }
+                       )
+  }
+,
+)
 
 
 class PortalFolder(DynamicType, CMFCatalogAware, Folder):
@@ -71,6 +78,8 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
     meta_type = 'Portal Folder'
     portal_type = 'Folder'
 
+    __implements__ = (DynamicType.__implements__, Folder.__implements__)
+
     security = ClassSecurityInfo()
 
     description = ''
@@ -82,23 +91,21 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
         self.id = id
         self.title = title
 
-    security.declareProtected( CMFCorePermissions.ManageProperties
-                             , 'setTitle')
+    security.declareProtected(ManageProperties, 'setTitle')
     def setTitle( self, title ):
         """
             Edit the folder title.
         """
         self.title = title
 
-    security.declareProtected( CMFCorePermissions.ManageProperties
-                             , 'setDescription')
+    security.declareProtected(ManageProperties, 'setDescription')
     def setDescription( self, description ):
         """
             Edit the folder description.
         """
         self.description = description
 
-    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
+    security.declareProtected(ManageProperties, 'edit')
     def edit(self, title='', description=''):
         """
         Edit the folder title (and possibly other attributes later)
@@ -220,8 +227,7 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
         return map( lambda item: item[1],
                     self._filteredItems( ids, filter ) )
 
-    security.declareProtected( ListFolderContents
-                             , 'listFolderContents' )
+    security.declareProtected(ListFolderContents, 'listFolderContents')
     def listFolderContents( self, spec=None, contentFilter=None ):
         """
             Hook around 'contentValues' to let 'folder_contents'
@@ -366,8 +372,12 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
     def checkIdAvailable(self, id):
         try:
             self._checkId(id)
+        except BadRequest:
+            return 0
         except:
-            if sys.exc_info()[0] == 'Bad Request':
+            # Zope < 2.7
+            e, v, tb = sys.exc_info(); del tb
+            if str(e) == 'Bad Request':
                 return 0
             raise  # Some other exception.
         else:
@@ -402,6 +412,7 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
         # This assists the version in OFS.CopySupport.
         # It enables the clipboard to function correctly
         # with objects created by a multi-factory.
+        securityChecksDone = 0
         if (hasattr(object, '__factory_meta_type__') and
             hasattr(self, 'all_meta_types')):
             mt = object.__factory_meta_type__
@@ -419,14 +430,16 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
                 if _checkPermission(permission_name,self):
                     if not validate_src:
                         # We don't want to check the object on the clipboard
-                        return
-                    try: parent = aq_parent(aq_inner(object))
-                    except: parent = None
-                    if getSecurityManager().validate(None, parent,
-                                                     None, object):
-                        # validation succeeded
-                        return
-                    raise 'Unauthorized', object.getId()
+                        securityChecksDone = 1
+                    else:
+                        try: parent = aq_parent(aq_inner(object))
+                        except: parent = None
+                        if getSecurityManager().validate(None, parent,
+                                                         None, object):
+                            # validation succeeded
+                            securityChecksDone = 1
+                        else:
+                            raise 'Unauthorized', object.getId()
                 else:
                     raise 'Unauthorized', permission_name
             #
@@ -444,18 +457,32 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
                     # Ensure the user is allowed to access the object on the
                     # clipboard.
                     if not validate_src:
-                        return
-                    try: parent = aq_parent(aq_inner(object))
-                    except: parent = None
-                    if getSecurityManager().validate(None, parent,
-                                                     None, object):
-                        return
-                    id = object.getId()
-                    raise 'Unauthorized', id
+                        securityChecksDone = 1
+                    else:
+                        try: parent = aq_parent(aq_inner(object))
+                        except: parent = None
+                        if getSecurityManager().validate(None, parent,
+                                                         None, object):
+                            securityChecksDone = 1
+                        else:
+                            raise 'Unauthorized', object.getId()
                 else:
                     raise 'Unauthorized', method_name
-        PortalFolder.inheritedAttribute(
-            '_verifyObjectPaste')(self, object, validate_src)
+
+        # Call OFS' _verifyObjectPaste if necessary
+        if not securityChecksDone:
+            PortalFolder.inheritedAttribute(
+                '_verifyObjectPaste')(self, object, validate_src)
+
+        # Finally, check allowed content types
+        if hasattr(aq_base(object), '_getPortalTypeName'):
+            contentType = object._getPortalTypeName()
+            if contentType is not None:
+                pt = getToolByName(self, 'portal_types')
+                myType = pt.getTypeInfo(self)
+                if myType is not None and not myType.allowType(contentType):
+                    raise ValueError, \
+                          "Disallowed to paste subobject type '%s'." % contentType
 
     security.setPermissionDefault(AddPortalContent, ('Owner','Manager'))
     security.setPermissionDefault(AddPortalFolders, ('Owner','Manager'))
@@ -474,7 +501,7 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
         """
         try:
             action = self.getTypeInfo().getActionById( 'mkdir' )
-        except TypeError:
+        except ValueError:
             self.invokeFactory( type_name='Folder', id=id )
         else:
             # call it
