@@ -1,46 +1,68 @@
-import Zope
 from unittest import TestCase, TestSuite, makeSuite, main
 
-from Products.CMFCore.TypesTool import\
-     FactoryTypeInformation as FTI,\
-     ScriptableTypeInformation as STI,\
-     TypesTool,Unauthorized
+import Zope
+try:
+    Zope.startup()
+except AttributeError:
+    # for Zope versions before 2.6.1
+    pass
+try:
+    from Interface.Verify import verifyClass
+except ImportError:
+    # for Zope versions before 2.6.0
+    from Interface import verify_class_implementation as verifyClass
 
+from types import DictType
+
+from AccessControl import Unauthorized
+from AccessControl.SecurityManagement import newSecurityManager
+from AccessControl.SecurityManagement import noSecurityManager
+from AccessControl.SecurityManager import setSecurityPolicy
+from Acquisition import aq_base
+from webdav.NullResource import NullResource
+from Products.PythonScripts.PythonScript import PythonScript
+from Products.PythonScripts.standard import url_quote
+
+from Products.CMFCore.ActionInformation import ActionInformation
+from Products.CMFCore.TypesTool import FactoryTypeInformation as FTI
+from Products.CMFCore.TypesTool import ScriptableTypeInformation as STI
+from Products.CMFCore.TypesTool import TypesTool
 from Products.CMFCore.PortalFolder import PortalFolder
 from Products.CMFCore.utils import _getViewFor
 
-from Products.CMFCore.tests.base.testcase import \
-     SecurityRequestTest
-from Products.CMFCore.tests.base.security import \
-     OmnipotentUser, UserWithRoles
-from Products.CMFCore.tests.base.dummy import \
-     DummyObject, addDummy, DummyTypeInfo,\
-     DummyFolder, DummyFTI
+from Products.CMFCore.tests.base.security import OmnipotentUser
+from Products.CMFCore.tests.base.security import UserWithRoles
+from Products.CMFCore.tests.base.dummy import DummyFactory
+from Products.CMFCore.tests.base.dummy import DummyFolder
+from Products.CMFCore.tests.base.dummy import DummyObject
+from Products.CMFCore.tests.base.dummy import DummySite
+from Products.CMFCore.tests.base.dummy import DummyUserFolder
+from Products.CMFCore.tests.base.tidata import FTIDATA_ACTIONS
+from Products.CMFCore.tests.base.tidata import FTIDATA_CMF13
+from Products.CMFCore.tests.base.tidata import FTIDATA_CMF13_FOLDER
+from Products.CMFCore.tests.base.tidata import FTIDATA_DUMMY
+from Products.CMFCore.tests.base.tidata import STI_SCRIPT
+from Products.CMFCore.tests.base.testcase import SecurityTest
 
-from AccessControl.SecurityManagement import newSecurityManager
-from AccessControl.SecurityManagement import noSecurityManager
 
-from Products.PythonScripts.standard import url_quote
-from webdav.NullResource import NullResource
-from Acquisition import aq_base
-
-class TypesToolTests( SecurityRequestTest ):
+class TypesToolTests(SecurityTest):
 
     def setUp( self ):
-        SecurityRequestTest.setUp(self)
-        root = self.root
-        root.addDummy = addDummy
-
-        root._setObject( 'portal_types', TypesTool() )
-        tool = root.portal_types
-        tool._setObject( 'Dummy Content', DummyFTI ) 
-    
+        SecurityTest.setUp(self)
+        self.site = DummySite('site').__of__(self.root)
+        self.acl_users = self.site._setObject( 'acl_users', DummyUserFolder() )
+        self.ttool = self.site._setObject( 'portal_types', TypesTool() )
+        fti = FTIDATA_DUMMY[0].copy()
+        del fti['id']
+        self.ttool._setObject( 'Dummy Content', apply( FTI, ('Dummy Content',), fti) )
+ 
     def test_processActions( self ):
         """
         Are the correct, permitted methods returned for actions?
         """
-        self.root._setObject( 'portal', PortalFolder( 'portal', '' ) )
-        portal = self.root.portal
+        site = self.site
+        portal = site._setObject( 'portal', PortalFolder(id='portal') )
+        portal.manage_addProduct = { 'FooProduct' : DummyFactory(portal) }
         portal.invokeFactory( 'Dummy Content', 'actions_dummy' )
         dummy = portal._getOb( 'actions_dummy' )
 
@@ -63,7 +85,7 @@ class TypesToolTests( SecurityRequestTest ):
         Test that everything returned by allMetaTypes can be
         traversed to.
         """
-        tool = self.root.portal_types
+        tool = self.ttool
         meta_types={}
         # Seems we get NullResource if the method couldn't be traverse to
         # so we check for that. If we've got it, something is b0rked.
@@ -71,13 +93,51 @@ class TypesToolTests( SecurityRequestTest ):
             meta_types[factype['name']]=1
             # The url_quote below is necessary 'cos of the one in
             # main.dtml. Could be removed once that is gone.
-            self.failIf(type(aq_base(tool.unrestrictedTraverse(url_quote(factype['action'])))) is NullResource)
+            act = tool.unrestrictedTraverse(url_quote(factype['action']))
+            self.failIf(type(aq_base(act)) is NullResource)
 
         # Check the ones we're expecting are there
         self.failUnless(meta_types.has_key('Scriptable Type Information'))
         self.failUnless(meta_types.has_key('Factory-based Type Information'))
 
-class TypeInfoTests( TestCase ):
+    def test_constructContent(self):
+        site = self.site
+        acl_users = self.acl_users
+        ttool = self.ttool
+        setSecurityPolicy(self._oldPolicy)
+        newSecurityManager(None, acl_users.all_powerful_Oz)
+        self.site._owner = (['acl_users'], 'all_powerful_Oz')
+        sti_baz = STI('Baz',
+                      permission='Add portal content',
+                      constructor_path='addBaz')
+        ttool._setObject('Baz', sti_baz)
+        ttool._setObject( 'addBaz',  PythonScript('addBaz') )
+        s = ttool.addBaz
+        s.write(STI_SCRIPT)
+
+        f = site._setObject( 'folder', PortalFolder(id='folder') )
+        f.manage_addProduct = { 'FooProduct' : DummyFactory(f) }
+        f._owner = (['acl_users'], 'user_foo')
+        self.assertEqual( f.getOwner(), acl_users.user_foo )
+
+        ttool.constructContent('Dummy Content', container=f, id='page1')
+        try:
+            ttool.constructContent('Baz', container=f, id='page2')
+        except Unauthorized:
+            self.fail('CMF Collector issue #165 (Ownership bug): '
+                      'Unauthorized raised' )
+
+def test_interface(self):
+        from Products.CMFCore.interfaces.portal_types \
+                import portal_types as ITypesTool
+        from Products.CMFCore.interfaces.portal_actions \
+                import ActionProvider as IActionProvider
+
+        verifyClass(ITypesTool, TypesTool)
+        verifyClass(IActionProvider, TypesTool)
+
+
+class TypeInfoTests(TestCase):
     
     def test_construction( self ):
         ti = self._makeInstance( 'Foo'
@@ -119,9 +179,10 @@ class TypeInfoTests( TestCase ):
     def test_GlobalHide( self ):
         self.tool = TypesTool()        
         tnf = self._makeAndSetInstance( 'Folder', filter_content_types=0)
-        taf = self._makeAndSetInstance( 'Allowing Folder',
-                                  allowed_content_types=('Hidden','Not Hidden'))
-        tih = self._makeAndSetInstance( 'Hidden'     ,global_allow=0)
+        taf = self._makeAndSetInstance( 'Allowing Folder'
+                                      , allowed_content_types=( 'Hidden'
+                                                              ,'Not Hidden'))
+        tih = self._makeAndSetInstance( 'Hidden', global_allow=0)
         tnh = self._makeAndSetInstance( 'Not Hidden')
         # make sure we're normally hidden but everything else is visible
         self.failIf     ( tnf.allowType( 'Hidden' ) )
@@ -131,9 +192,12 @@ class TypeInfoTests( TestCase ):
         self.failUnless ( taf.allowType( 'Not Hidden') )
         # make sure we're available in a non-content-type-filtered type
         # where we have been explicitly allowed
-        taf2 = self._makeAndSetInstance( 'Allowing Folder2',
-                                   allowed_content_types=('Hidden','Not Hidden'),
-                                   filter_content_types=0)
+        taf2 = self._makeAndSetInstance( 'Allowing Folder2'
+                                       , allowed_content_types=( 'Hidden'
+                                                               , 'Not Hidden'
+                                                               )
+                                       , filter_content_types=0
+                                       )
         self.failUnless ( taf2.allowType( 'Hidden' ) )
         self.failUnless ( taf2.allowType( 'Not Hidden') )
         
@@ -145,63 +209,28 @@ class TypeInfoTests( TestCase ):
         ti = self._makeInstance( 'Foo', allow_discussion=1 )
         self.failUnless( ti.allowDiscussion() )
 
-    ACTION_LIST = \
-    ( { 'id'            : 'view'
-      , 'name'          : 'View'
-      , 'action'        : 'foo_view'
-      , 'permissions'   : ( 'View', )
-      , 'category'      : 'object'
-      , 'visible'       : 1
-      }
-    , { 'name'          : 'Edit'                # Note: No ID passed
-      , 'action'        : 'foo_edit'
-      , 'permissions'   : ( 'Modify', )
-      , 'category'      : 'object'
-      , 'visible'       : 1
-      }
-    , { 'name'          : 'Object Properties'   # Note: No ID passed
-      , 'action'        : 'foo_properties'
-      , 'permissions'   : ( 'Modify', )
-      , 'category'      : 'object'
-      , 'visible'       : 1
-      }
-    , { 'id'            : 'slot'
-      , 'action'        : 'foo_slot'
-      , 'category'      : 'object'
-      , 'visible'       : 0
-      }
-    )
-
-    def _ripActionValues( self, key, actions ):
-        return filter( None, map( lambda x, key=key: x.get( key, None )
-                                , actions
-                                ) )
-
     def test_listActions( self ):
         ti = self._makeInstance( 'Foo' )
-        self.failIf( ti.getActions() )
+        self.failIf( ti.listActions() )
 
-        ti = self._makeInstance( 'Foo', actions=self.ACTION_LIST )
-
-        actions = ti.getActions()
+        ti = self._makeInstanceByFTIData(FTIDATA_ACTIONS)
+        actions = ti.listActions()
         self.failUnless( actions )
 
-        ids = self._ripActionValues( 'id', actions )
+        ids = [ x.getId() for x in actions ]
         self.failUnless( 'view' in ids )
         self.failUnless( 'edit' in ids )
         self.failUnless( 'objectproperties' in ids )
         self.failUnless( 'slot' in ids )
 
-        names = self._ripActionValues( 'name', actions )
+        names = [ x.Title() for x in actions ]
         self.failUnless( 'View' in names )
         self.failUnless( 'Edit' in names )
         self.failUnless( 'Object Properties' in names )
         self.failIf( 'slot' in names )
-        self.failIf( 'Slot' in names )
+        self.failUnless( 'Slot' in names )
         
-        visible = filter( None, map( lambda x:
-                                        x.get( 'visible', 0 ) and x['id']
-                                   , actions ) )
+        visible = [ x.getId() for x in actions if x.getVisibility() ]
         self.failUnless( 'view' in visible )
         self.failUnless( 'edit' in visible )
         self.failUnless( 'objectproperties' in visible )
@@ -213,15 +242,15 @@ class TypeInfoTests( TestCase ):
         marker = []
         self.assertEqual( id( ti.getActionById( 'view', marker ) )
                         , id( marker ) )
-        self.assertRaises( TypeError, ti.getActionById, 'view' )
+        self.assertRaises( ValueError, ti.getActionById, 'view' )
 
-        ti = self._makeInstance( 'Foo', actions=self.ACTION_LIST )
+        ti = self._makeInstanceByFTIData(FTIDATA_ACTIONS)
         self.assertEqual( id( ti.getActionById( 'foo', marker ) )
                         , id( marker ) )
-        self.assertRaises( TypeError, ti.getActionById, 'foo' )
+        self.assertRaises( ValueError, ti.getActionById, 'foo' )
         
         action = ti.getActionById( 'view' )
-        self.assertEqual( action, 'foo_view' )
+        self.assertEqual( action, '' )
         
         action = ti.getActionById( 'edit' )
         self.assertEqual( action, 'foo_edit' )
@@ -231,13 +260,78 @@ class TypeInfoTests( TestCase ):
         
         action = ti.getActionById( 'slot' )
         self.assertEqual( action, 'foo_slot' )
-        
+
+    def _checkContentTI(self, ti):
+        wanted_actions_text0 = 'string:${object_url}/dummy_view'
+        wanted_actions_text1 = 'string:${object_url}/dummy_edit_form'
+        wanted_actions_text2 = 'string:${object_url}/metadata_edit_form'
+
+        self.failUnless( isinstance( ti._actions[0], ActionInformation ) )
+        self.assertEqual( len( ti._actions ), 3 )
+        self.assertEqual(ti._actions[0].action.text, wanted_actions_text0)
+        self.assertEqual(ti._actions[1].action.text, wanted_actions_text1)
+        self.assertEqual(ti._actions[2].action.text, wanted_actions_text2)
+
+        action0 = ti._actions[0]
+        self.assertEqual( action0.getId(), 'view' )
+        self.assertEqual( action0.Title(), 'View' )
+        self.assertEqual( action0.getActionExpression(), wanted_actions_text0 )
+        self.assertEqual( action0.getCondition(), '' )
+        self.assertEqual( action0.getPermissions(), ( 'View', ) )
+        self.assertEqual( action0.getCategory(), 'object' )
+        self.assertEqual( action0.getVisibility(), 1 )
+
+    def _checkFolderTI(self, ti):
+        wanted_actions_text0 = 'string:${object_url}'
+        wanted_actions_text1 = 'string:${object_url}/dummy_edit_form'
+        wanted_actions_text2 = 'string:${object_url}/folder_localrole_form'
+
+        self.failUnless( isinstance( ti._actions[0], ActionInformation ) )
+        self.assertEqual( len( ti._actions ), 3 )
+        self.assertEqual(ti._actions[0].action.text, wanted_actions_text0)
+        self.assertEqual(ti._actions[1].action.text, wanted_actions_text1)
+        self.assertEqual(ti._actions[2].action.text, wanted_actions_text2)
+
+    def test_CMF13_content_migration(self):
+
+        # use old FTI Data
+        ti = self._makeInstanceByFTIData(FTIDATA_CMF13)
+        self._checkContentTI(ti)
+
+        # simulate old FTI
+        ti._actions = FTIDATA_CMF13[0]['actions']
+        self.failUnless( isinstance( ti._actions[0], DictType ) )
+
+        # migrate FTI
+        ti.listActions()
+        self._checkContentTI(ti)
+
+    def test_CMF13_folder_migration(self):
+
+        # use old FTI Data
+        ti = self._makeInstanceByFTIData(FTIDATA_CMF13_FOLDER)
+        self._checkFolderTI(ti)
+
+        # simulate old FTI
+        ti._actions = FTIDATA_CMF13_FOLDER[0]['actions']
+        self.failUnless( isinstance( ti._actions[0], DictType ) )
+
+        # migrate FTI
+        ti.listActions()
+        self._checkFolderTI(ti)
+
 
 class FTIDataTests( TypeInfoTests ):
 
     def _makeInstance( self, id, **kw ):
         return apply( FTI, ( id, ), kw )
 
+    def _makeInstanceByFTIData(self, ftidata):
+        fti = ftidata[0].copy()
+        id = fti['id']
+        del fti['id']
+        return apply( FTI, ( id, ), fti )
+
     def test_properties( self ):
         ti = self._makeInstance( 'Foo' )
         self.assertEqual( ti.product, '' )
@@ -250,12 +344,24 @@ class FTIDataTests( TypeInfoTests ):
         self.assertEqual( ti.product, 'FooProduct' )
         self.assertEqual( ti.factory, 'addFoo' )
 
+    def test_interface(self):
+        from Products.CMFCore.interfaces.portal_types \
+                import ContentTypeInformation as ITypeInformation
+
+        verifyClass(ITypeInformation, FTI)
+        
 
 class STIDataTests( TypeInfoTests ):
 
     def _makeInstance( self, id, **kw ):
         return apply( STI, ( id, ), kw )
 
+    def _makeInstanceByFTIData(self, ftidata):
+        fti = ftidata[0].copy()
+        id = fti['id']
+        del fti['id']
+        return apply( STI, ( id, ), fti )
+
     def test_properties( self ):
         ti = self._makeInstance( 'Foo' )
         self.assertEqual( ti.permission, '' )
@@ -268,7 +374,14 @@ class STIDataTests( TypeInfoTests ):
         self.assertEqual( ti.permission, 'Add Foos' )
         self.assertEqual( ti.constructor_path, 'foo_add' )
 
-class FTIConstructionTests( TestCase ):
+    def test_interface(self):
+        from Products.CMFCore.interfaces.portal_types \
+                import ContentTypeInformation as ITypeInformation
+
+        verifyClass(ITypeInformation, STI)
+        
+
+class FTIConstructionTests(TestCase):
 
     def setUp( self ):
         noSecurityManager()
@@ -312,7 +425,7 @@ class FTIConstructionTests( TestCase ):
 
         self.failIf( ti.isConstructionAllowed( folder ) )
 
-class FTIConstructionTests_w_Roles( TestCase ):
+class FTIConstructionTests_w_Roles(TestCase):
 
     def tearDown( self ):
         noSecurityManager()
